# Unit tests
Unit testing is implemented by utilizing [Check](https://libcheck.github.io/check/). It is tightly coupled with
the source code and integratd into GNU Autotools build system. See `../README.md`
for more info.

# Manual tests
For manual testing of `Sentinel-Minipots` there is a small utility `dev_proxy`
in [Sentinel:Proxy repository](https://gitlab.nic.cz/turris/sentinel/proxy)
which purpose is to print all captured communication with local
ZMQ socket to `stdout`. So one can observe the output generated by `Minipots`.

# Integration tests
Integration test framework (in folder `integration`) serves for overall automated
testing of `Minipots` component and minipot pipeline as a whole system.
It implements counterparts for the pipeline from both sides.
It emulates attackers and captures data generated by `Minipots` component
from oposite end of the pipeline. It compares captured output from the pipeline
with intended outputs.

Pipeline can be composed from Minipots only or can contain more
components but at least Proxy, Mosquitto and Smash.
However, it is recomened to tests it together with Sanity check.

## How to run tests?
The integration tests are run by simply creating and running a script file
containing defined tests. This allows ultimate modularity. Everyone can create
their own script files with any tests. Currently, there are 2 script files for
each type of Minipot. Scripts `run_xxx_from_minipots` perform tests where Minipots
outputs are captured directly from Minipots. This is usefull in earlier stages
of the development process. Scripts `run_xxx_from_sanity_check` perform tests
where Minipots outputs are captured from Sanity check. This is usefull to
verify that Sentinel messages generated by Minipots pass Sentinel server
infrastructure input checks and can be later processed or enriched by more
information.

For the best setup see Test setup part of this document.

## Dependencies
- pyzmq
- msgpack

# Test setup
Manual and integration tests requires running `Minipots` instance (see `../README.md`).
For the fastest testing it is possible to run multiple `Minipots` instances
at the same time. However, the instances must be run on mutually distinct
ports and ZMQ sockets. See Example of test setup.

**It is desirable to run `Minipot` component in `Valgrind` environment
especially in `Memcheck` for catching hidden memory flaws which could cause
security issues.**

For running `Minipots` in `Valgrind` prepend its command with:
```
valgrind \
--leak-check=full --trace-children=yes \
--show-leak-kinds=definite,indirect,possible --track-fds=yes \
--error-exitcode=1 --track-origins=yes \
```

# Example of test setup
Script `run_minipots_for_testing.sh` runs `Minipots` instance
on predefined ports and predefined ZMQ socket based on input arguments.
It has two arguments. The first argument is test type. There are short and long
versions of test types. Test type value is either `m`|`manual` / `i`|`integration` /
`p`|`pipeline` / `t`|`throughput`. If `integration` or `manual` test type
selected, the second script argument must be also defined. These two
configurations are meant for testing `Minipots` component itself.
`pipeline` test type is desired for testing Minipot pipeline composed from more
Sentinel components not only `Minipots`. `throughput` test type is intended for
testing of ZMQ behaviour and its limit. The second argument is minipot type.
There are again short and long versions. Its value is either `f`|`ftp` /
`h`|`http` / `s`|`smtp` / `t`|`telnet`.
